@@ -96,8 +96,8 @@ class DenseBase(Operator):
             if naxesextra < 0:
                 raise ValueError(
                     "The number of input and output dimensions ('{0}' and '{1}"
-                    "')exceeds the number of dimensions of the input array {2}"
-                    ".".format(naxesin, naxesout, data.ndim))
+                    "') exceeds the number of dimensions of the input array {2"
+                    "}.".format(naxesin, naxesout, data.ndim))
             naxesextra = data.ndim - naxesin - naxesout
         if naxesin + naxesout + naxesextra != data.ndim:
             raise ValueError(
@@ -210,7 +210,7 @@ class DenseBlockDiagonalOperator(DenseBase):
             if broadcast_shapes(extrashapein, self._sl) != extrashapein:
                 raise NotImplementedError(
                     'With this input shape, the operator is not diagonal block'
-                    ' anymore. Its transpose would not be incorrect.')
+                    ' anymore. Its transpose would not be correct.')
         self.set_rule(('.', DenseBlockDiagonalOperator), self._rule_mul,
                       CompositionOperator)
 
@@ -584,6 +584,7 @@ class Rotation2dOperator(DenseBlockDiagonalOperator):
         angle = np.asarray(angle)
         if dtype is None:
             dtype = float_dtype(angle.dtype)
+        angle = np.asarray(angle, dtype)
         if degrees:
             angle = np.radians(angle)
         cosa = np.cos(angle)
@@ -659,9 +660,9 @@ class Rotation3dOperator(DenseBlockDiagonalOperator):
         if dtype is None:
             dtype = np.find_common_type([float_dtype(a.dtype)
                                          for a in (a1, a2, a3)], [])
-        a1 = a1.astype(dtype)
-        a2 = a2.astype(dtype)
-        a3 = a3.astype(dtype)
+        a1 = np.asarray(a1, dtype)
+        a2 = np.asarray(a2, dtype)
+        a3 = np.asarray(a3, dtype)
         if degrees:
             a1 = np.radians(a1)
             a2 = np.radians(a2)
