@@ -6,20 +6,22 @@ import multiprocessing
 import numpy as np
 import operator
 import os
-import scipy.sparse
 import signal
 import types
 
 from contextlib import contextmanager
 from itertools import izip
 from . import cythonutils as cu
+from ..warnings import warn, PyOperatorsDeprecationWarning
 
 __all__ = ['all_eq',
+           'broadcast_shapes',
            'cast',
            'complex_dtype',
            'first',
            'first_is_not',
            'float_dtype',
+           'groupbykey',
            'ifirst',
            'ifirst_is_not',
            'inspect_special_values',
@@ -28,6 +30,7 @@ __all__ = ['all_eq',
            'isalias',
            'isclassattr',
            'isscalar',
+           'isscalarlike',
            'izip_broadcast',
            'least_greater_multiple',
            'merge_none',
@@ -39,6 +42,7 @@ __all__ = ['all_eq',
            'pi',
            'product',
            'renumerate',
+           'reshape_broadcast',
            'strelapsed',
            'strenum',
            'strinfo',
@@ -81,7 +85,7 @@ def all_eq(a, b):
         for a_, b_ in izip(a, b):
             if not all_eq(a_, b_):
                 return False
-            return True
+        return True
     if isinstance(a, types.MethodType):
         if type(a) is not type(b):
             return False
@@ -93,6 +97,40 @@ def all_eq(a, b):
     return a == b
 
 
+def broadcast_shapes(*shapes):
+    """
+    Broadcast any number of shapes against each other.
+
+    Parameters
+    ----------
+    *shapes : tuples
+        The shapes to broadcast
+
+    Example
+    -------
+    >>> broadcast_shapes((1,5), (3, 2, 1))
+    (3, 2, 5)
+
+    """
+    if any(not isinstance(s, tuple) for s in shapes):
+        raise TypeError('The input shapes are not tuples.')
+    ndim = max(len(s) for s in shapes)
+    shapes_ = [(ndim-len(s)) * [1] + list(s) for s in shapes]
+    outshape = []
+    for idim, dims in enumerate(zip(*shapes_)):
+        dims = [dim for dim in dims if dim != 1]
+        if len(dims) == 0:
+            d = 1
+        elif any(dim != dims[0] for dim in dims):
+            raise ValueError(
+                'The shapes could not be broadcast together {}'.format(
+                    ' '.join(str(s) for s in shapes)))
+        else:
+            d = dims[0]
+        outshape.append(d)
+    return tuple(outshape)
+
+
 def cast(arrays, dtype=None, order='c'):
     """
     Cast a list of arrays into a same data type.
@@ -177,7 +215,7 @@ def float_dtype(dtype):
     return dtype
 
 
-def first(l, f):
+def first(l, f, reverse=False):
     """
     Return first item in list that verifies a certain condition, or raise
     a ValueError exception otherwise.
@@ -188,20 +226,27 @@ def first(l, f):
         List of elements to be searched for.
     f : function
         Function that evaluates to True to match an element.
+    reverse : boolean
+        True for reverse search.
 
     Example:
     --------
     >>> first([1.,2.,3.], lambda x: x > 1.5)
-    2.
+    2.0
+    >>> first([1.,2.,3.], lambda x: x > 1.5, reverse=True)
+    3.0
 
     """
+    if reverse:
+        return first(reversed(tuple(l)), f)
+
     try:
         return next((_ for _ in l if f(_)))
     except StopIteration:
         raise ValueError('There is no matching item in the list.')
 
 
-def first_is_not(l, v):
+def first_is_not(l, v, reverse=False):
     """
     Return first item in list which is not the specified value.
     If all items are the specified value, return it.
@@ -212,17 +257,43 @@ def first_is_not(l, v):
         The list of elements to be inspected.
     v : object
         The value not to be matched.
+    reverse : boolean
+        True for reverse search.
 
     Example:
     --------
     >>> first_is_not(['a', 'b', 'c'], 'a')
     'b'
+    >>> first_is_not(['a', 'b', 'c'], 'b', reverse=True)
+    'c'
 
     """
+    if reverse:
+        return first_is_not(reversed(tuple(l)), v)
     return next((_ for _ in l if _ is not v), v)
 
 
-def ifirst(l, match):
+def groupbykey(iterable, key):
+    """
+    Create an iterator which returns (key, sub-iterator) grouped by each
+    value of key.
+
+    """
+    iterator = izip(iterable, key)
+    i, value = next(iterator)
+    l = [i]
+    for i, k in iterator:
+        if k == value:
+            l.append(i)
+            continue
+        yield value, l
+        value = k
+        l = [i]
+    if len(l) != 0:
+        yield value, l
+
+
+def ifirst(l, match, reverse=False):
     """
     Return the index of the first item in a list that verifies a certain
     condition or is equal to a certain value. Raise a ValueError exception
@@ -230,20 +301,29 @@ def ifirst(l, match):
 
     Parameters
     ----------
-    l : list
+    l : iterator
         List of elements to be searched for.
     match : callable or object
         Function that evaluates to True to match an element or the element
         to be matched.
+    reverse : boolean
+        True for reverse search.
 
     Example:
     --------
     >>> ifirst([1.,2.,3.], lambda x: x > 1.5)
     1
+    >>> ifirst([1.,2.,3.], lambda x: x > 1.5, reverse=True)
+    2
     >>> ifirst([1., 2., 3.], 2)
     1
 
     """
+    if reverse:
+        l = tuple(l)
+        index = ifirst(reversed(l), match)
+        return len(l) - index - 1
+
     try:
         if not callable(match):
             return next((i for i, _ in enumerate(l) if _ == match))
@@ -252,7 +332,7 @@ def ifirst(l, match):
         raise ValueError('There is no matching item in the list.')
 
 
-def ifirst_is_not(l, v):
+def ifirst_is_not(l, v, reverse=False):
     """
     Return index of first item in list which is not the specified value.
     If the list is empty or if all items are the specified value, raise
@@ -264,13 +344,21 @@ def ifirst_is_not(l, v):
         The list of elements to be inspected.
     v : object
         The value not to be matched.
+    reverse : boolean
+        True for reverse search.
 
     Example:
     --------
     >>> ifirst_is_not(['a', 'b', 'c'], 'a')
     1
+    >>> ifirst_is_not(['a', 'b', 'c'], 'a', reverse=True)
+    2
 
     """
+    if reverse:
+        l = tuple(l)
+        index = ifirst_is_not(reversed(l), v)
+        return len(l) - index - 1
     try:
         return next((i for i, _ in enumerate(l) if _ is not v))
     except StopIteration:
@@ -351,15 +439,16 @@ def isclassattr(cls, a):
     return False
 
 
-def isscalar(data):
-    """Hack around np.isscalar oddity"""
-    if isinstance(data, np.ndarray):
-        return data.ndim == 0
-    if isinstance(data, (str, unicode)):
-        return True
-    if isinstance(data, (collections.Container, scipy.sparse.base.spmatrix)):
-        return False
-    return True
+def isscalar(x):
+    """Deprecated."""
+    warn('Use isscalarlike instead of isscalar.',
+         PyOperatorsDeprecationWarning)
+    return isscalarlike(x)
+
+
+def isscalarlike(x):
+    """Return True for scalars and 0-ranked arrays."""
+    return np.isscalar(x) or isinstance(x, np.ndarray) and x.ndim == 0
 
 
 def izip_broadcast(*args):
@@ -490,6 +579,46 @@ def renumerate(l):
     return izip(xrange(len(l)-1, -1, -1), reversed(l))
 
 
+def reshape_broadcast(x, shape):
+    """
+    Reshape an array by setting broadcastable dimensions' strides to zero.
+
+    Parameters
+    ----------
+    x : array-like
+        The array to be reshaped.
+    shape : tuple of int
+        New shape of array. It can be any positive number along the axes of x
+        of length 1.
+
+    Example
+    -------
+    >>> a = np.arange(3).reshape((3, 1))
+    >>> b = reshape_broadcast(a, (2, 3, 2))
+    >>> print(b)
+    [[[0 0]
+      [1 1]
+      [2 2]]
+
+     [[0 0]
+      [1 1]
+      [2 2]]]
+    >>> b.shape
+    (2, 3, 2)
+    >>> b.strides
+    (0, 8, 0)
+
+    """
+    x = np.asanyarray(x)
+    if len(shape) < x.ndim or \
+       any(os != 1 and os != ns for os, ns in zip(x.shape, shape[-x.ndim:])):
+        raise ValueError("The requested shape '{0}' is incompatible with that "
+                         "of the array '{1}'.".format(shape, x.shape))
+    strides = (len(shape) - x.ndim) * (0,) + tuple(
+              (0 if sh == 1 else st for sh, st in zip(x.shape, x.strides)))
+    return np.lib.stride_tricks.as_strided(x, shape, strides)
+
+
 def strelapsed(t0, msg='Elapsed time'):
     """
     Return an information message including elapsed time.
@@ -527,7 +656,7 @@ def strenum(choices, last='or'):
 
     Examples
     --------
-    >>> strenum(['blue', 'red', 'yellow'], 'or')
+    >>> strenum(['blue', 'red', 'yellow'])
     "'blue', 'red' or 'yellow'"
 
     """
