@@ -4,10 +4,11 @@ import numpy as np
 from numpy.testing import assert_equal
 from pyoperators import Operator
 from pyoperators.utils import (
-    cast, complex_dtype, first, float_dtype, ifirst, first_is_not,
-    ifirst_is_not, inspect_special_values, interruptible, isscalar,
-    izip_broadcast, least_greater_multiple, one, pi, product, strenum,
-    strplural, strshape, uninterruptible, zero)
+    broadcast_shapes, cast, complex_dtype, first, float_dtype, ifirst,
+    first_is_not, ifirst_is_not, groupbykey, inspect_special_values,
+    interruptible, isscalarlike, izip_broadcast, least_greater_multiple, one,
+    pi, product, reshape_broadcast, strenum, strplural, strshape,
+    uninterruptible, zero)
 from pyoperators.utils.testing import assert_eq, assert_raises, assert_same
 
 dtypes = [np.dtype(t) for t in (np.bool8, np.uint8, np.int8, np.uint16,
@@ -21,12 +22,30 @@ def assert_dtype(a, d):
     assert_eq(a.dtype, d)
 
 
-def assert_is_scalar(o):
-    assert isscalar(o)
+def test_broadcast_shapes():
+    shapes = [((),), ((), ()), ((), (), ()),
+              ((1,),), ((), (1,)), ((), (1,), (1,)),
+              ((2,),), ((), (2,)), ((1,), (2,)),
+              ((), (2,), (2,)), ((), (2,), (1,)), ((2,), (1,), ()),
+              ((1,), (2, 1)), ((), (2, 1)), ((1,), (2, 1), ()),
+              ((1,), (1, 2)), ((), (1, 2)), ((1,), (1, 2), ()),
+              ((2,), (2, 1)), ((), (2,), (2, 1)), ((2,), (2, 1), ()),
+              ((1, 2), (2, 1)), ((), (1, 2), (2, 1)), ((1, 2), (2, 1), ()),
+              ((1, 1, 4), (1, 3, 1), (2, 1, 1), (), (1, 1, 1))]
+    expecteds = [(), (), (),
+                 (1,), (1,), (1,),
+                 (2,), (2,), (2,),
+                 (2,), (2,), (2,),
+                 (2, 1), (2, 1), (2, 1),
+                 (1, 2), (1, 2), (1, 2),
+                 (2, 2), (2, 2), (2, 2),
+                 (2, 2), (2, 2), (2, 2),
+                 (2, 3, 4)]
 
-
-def assert_is_not_scalar(o):
-    assert not isscalar(o)
+    def func(shape, expected):
+        assert_equal(broadcast_shapes(*shape), expected)
+    for shape, expected in zip(shapes, expecteds):
+        yield func, shape, expected
 
 
 def test_cast():
@@ -81,26 +100,32 @@ def test_float_dtype():
 
 def test_first1():
     assert first([1, 2, 3], lambda x: x > 1.5) == 2
+    assert first([1, 2, 3], lambda x: x > 1.5, reverse=True) == 3
 
 
 def test_first2():
     assert_raises(ValueError, first, [1, 2, 3], lambda x: x > 3)
+    assert_raises(ValueError, first, [1, 2, 3], lambda x: x > 3, reverse=True)
 
 
 def test_ifirst1():
     assert ifirst([1, 2, 3], lambda x: x > 1.5) == 1
+    assert ifirst([1, 2, 3], lambda x: x > 1.5, reverse=True) == 2
 
 
 def test_ifirst2():
     assert_raises(ValueError, ifirst, [1, 2, 3], lambda x: x > 3)
+    assert_raises(ValueError, ifirst, [1, 2, 3], lambda x: x > 3, reverse=True)
 
 
 def test_ifirst3():
-    assert ifirst([1, 2, 3], 2.) == 1
+    assert ifirst([1, 2, 2, 3], 2.) == 1
+    assert ifirst([1, 2, 2, 3], 2., reverse=True) == 2
 
 
 def test_ifirst4():
     assert_raises(ValueError, ifirst, [1, 2, 3], 4)
+    assert_raises(ValueError, ifirst, [1, 2, 3], 4, reverse=True)
 
 
 def test_first_is_not():
@@ -109,12 +134,30 @@ def test_first_is_not():
     assert first_is_not([], None) is None
     assert first_is_not([None, None], None) is None
 
+    assert first_is_not([1, 2], 2, reverse=True) == 1
+    assert first_is_not([{}, None, None], None, reverse=True) == {}
+    assert first_is_not([], None, reverse=True) is None
+    assert first_is_not([None, None], None, reverse=True) is None
+
 
 def test_ifirst_is_not():
-    assert ifirst_is_not([1, 2], 1) == 1
-    assert ifirst_is_not([None, None, {}], None) == 2
-    assert_raises(ValueError, ifirst_is_not, [], None)
-    assert_raises(ValueError, ifirst_is_not, [None, None], None)
+    assert ifirst_is_not([1, 2], 2, reverse=True) == 0
+    assert ifirst_is_not([{}, None, None], None, reverse=True) == 0
+    assert_raises(ValueError, ifirst_is_not, [], None, reverse=True)
+    assert_raises(ValueError, ifirst_is_not, [None, None], None, reverse=True)
+
+
+def test_groupbykey():
+    vals = ['a', 'b', 'c', 'd']
+    keys = itertools.combinations_with_replacement([1, 2, 3, 4], 4)
+
+    def func(k):
+        result = list(groupbykey(vals, k))
+        expected = [(k, tuple(i[0] for i in it)) for k, it in
+                    itertools.groupby(zip(vals, k), lambda x: x[1])]
+        assert_equal(result, expected)
+    for k in keys:
+        yield func, k
 
 
 def test_inspect_special_values():
@@ -170,13 +213,18 @@ def test_interruptible():
 
 
 def test_is_scalar():
-    for o in (object, True, 1, 1., np.array(1), np.int8, slice, Operator()):
-        yield assert_is_scalar, o
+    def func(x):
+        assert isscalarlike(x)
+    for x in (True, 1, 1., 'lkj', u'jj', np.array(1)):
+        yield func, x
 
 
 def test_is_not_scalar():
-    for o in ([], (), np.ones(1), np.ones(2)):
-        yield assert_is_not_scalar, o
+    def func(x):
+        assert not isscalarlike(x)
+    for x in ([], (), np.ones((0, 1)), np.ones(1), np.ones(2), object, np.int8,
+              slice, Operator()):
+        yield func, x
 
 
 def test_izip_broadcast1():
@@ -252,6 +300,26 @@ def test_product():
         yield func, product(o)
 
 
+def test_reshape_broadcast():
+    data = np.arange(20)
+    shapes = (4, 5), (1, 4, 5), (4, 1, 5), (4, 5, 1)
+    new_shapess = (
+        ((4, 5), (1, 4, 5), (2, 4, 5), (2, 3, 4, 5)),
+        ((1, 4, 5), (2, 4, 5), (1, 2, 4, 5), (2, 2, 4, 5), (2, 3, 2, 4, 5)),
+        ((4, 1, 5), (4, 2, 5), (1, 4, 2, 5), (2, 4, 2, 5), (2, 3, 4, 2, 5)),
+        ((4, 5, 1), (4, 5, 2), (1, 4, 5, 2), (2, 4, 5, 2), (2, 3, 4, 5, 2)))
+
+    def func(shape, new_shape):
+        data_ = data.reshape(shape)
+        expected = np.empty(new_shape)
+        expected[...] = data_
+        actual = reshape_broadcast(data_, new_shape)
+        assert_equal(actual, expected)
+    for shape, new_shapes in zip(shapes, new_shapess):
+        for new_shape in new_shapes:
+            yield func, shape, new_shape
+
+
 def test_strenum():
     assert_eq(strenum(['blue', 'red', 'yellow'], 'or'),
               "'blue', 'red' or 'yellow'")
